@@ -14,7 +14,7 @@ from fastmcp.client.transports import StreamableHttpTransport
 from agentmake.plugins.uba.lib.BibleBooks import BibleBooks
 from agentmake.plugins.uba.lib.BibleParser import BibleVerseParser
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, extractText, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_TEXT_EDITOR
-from agentmake.utils.files import searchFolder, isExistingPath
+from agentmake.utils.files import searchFolder, isExistingPath, sanitize_filename
 from agentmake.etextedit import launch_async
 from agentmake.utils.manage_package import getPackageLatestVersion
 from rich.console import Console
@@ -31,7 +31,7 @@ if not USER_OS == "Windows":
     import readline  # for better input experience
 
 # set window title
-set_title(f"BibleMate AI")
+set_title("BibleMate AI")
 
 parser = argparse.ArgumentParser(description = f"""BibleMate AI {BIBLEMATE_VERSION} CLI options""")
 # global options
@@ -43,9 +43,8 @@ parser.add_argument("-pe", "--promptengineer", action="store", dest="promptengin
 parser.add_argument("-s", "--steps", action="store", dest="steps", type=int, help="Specify the maximum number of steps allowed.")
 parser.add_argument("-e", "--exit", action="store_true", dest="exit", help="exit after the first response (for single-turn use cases).")
 # mcp options
-parser.add_argument("-t", "--token", action="store", dest="token", help="specify a static token to use for authentication with the MCP server; applicable to command `biblemate` only")
-parser.add_argument("-mcp", "--mcp", action="store", dest="mcp", help=f"specify a custom MCP server to use, e.g. 'http://127.0.0.1:{config.mcp_port}/mcp/'; applicable to command `biblemate` only")
-parser.add_argument("-p", "--port", action="store", dest="port", help=f"specify a port for the MCP server to use, e.g. {config.mcp_port}; applicable to command `biblematemcp` only")
+parser.add_argument("-t", "--token", action="store", dest="token", help="specify a static token to use for authentication with the MCP server")
+parser.add_argument("-mcp", "--mcp", action="store", dest="mcp", help=f"specify a custom MCP server to use, e.g. 'http://127.0.0.1:{config.mcp_port}/mcp/'")
 args = parser.parse_args()
 
 if not sys.stdin.isatty():
@@ -61,16 +60,20 @@ with open(CONFIG_FILE_BACKUP, "a", encoding="utf-8") as fileObj:
     fileObj.write(f'''\nconfig.backend="{config.backend}"''')
 
 AGENTMAKE_ENV_PATH = os.path.join(AGENTMAKE_USER_DIR, "agentmake.env")
-if config.backend == "googleai" and not os.getenv("GOOGLEAI_API_KEY") and os.path.isfile(AGENTMAKE_ENV_PATH):
+if config.backend == "googleai" and not os.getenv("GOOGLEAI_API_KEY"):
     googleai_api_key = DIALOGS.getInputDialog_sync(title="Google AI API key", text="Enter your Google AI API key:")
-    if googleai_api_key.strip():
+    if googleai_api_key and googleai_api_key.strip():
         googleai_api_key = googleai_api_key.strip()
         agentmake_env_content = readTextFile(AGENTMAKE_ENV_PATH)
-        if "\nGOOGLEAI_API_KEY=" in agentmake_env_content:
+        if os.path.isfile(AGENTMAKE_ENV_PATH) and "\nGOOGLEAI_API_KEY=" in agentmake_env_content:
             writeTextFile(AGENTMAKE_ENV_PATH, re.sub("\nGOOGLEAI_API_KEY=[^\n]*?\n", f'\nGOOGLEAI_API_KEY="{googleai_api_key}"\n', agentmake_env_content))
         else:
             with open(AGENTMAKE_ENV_PATH, "a", encoding="utf-8") as fileObj:
                 fileObj.write(f'''\nGOOGLEAI_API_KEY="{googleai_api_key}"\n''')
+        print("""###   Configuration Updated   ###
+Enter `biblemate` again to start using BibleMate AI with Google AI backend.
+""")
+        exit()
 
 AGENTMAKE_CONFIG["backend"] = config.backend
 DEFAULT_SYSTEM = "You are BibleMate AI, an autonomous agent designed to assist users with their Bible study."
@@ -80,6 +83,14 @@ Please provide me with the final answer to my original request based on the work
 
 # Original Request
 """
+TOOL_INSTRUCTION_PROMPT = """Please transform the following suggestions into clear, precise, and actionable instructions."""
+TOOL_INSTRUCTION_SUFFIX = """
+
+# Remember
+
+* Provide me with the instructions directly.
+* Do not start your response, like, 'Here are the insturctions ...'
+* Do not ask me if I want to execute the instruction."""
 
 # other temporary config changes
 if args.light == "true":
@@ -99,13 +110,6 @@ elif args.promptengineer == "false":
 if args.steps:
     config.max_steps = args.steps
 
-def mcp():
-    builtin_mcp_server = os.path.join(os.path.dirname(os.path.realpath(__file__)), "bible_study_mcp.py")
-    user_mcp_server = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "bible_study_mcp.py") # The user path has the same basename as the built-in one; users may copy the built-in server settings to this location for customization. 
-    mcp_script = readTextFile(user_mcp_server if os.path.isfile(user_mcp_server) else builtin_mcp_server)
-    mcp_script = mcp_script.replace("mcp.run(show_banner=False)", f'''mcp.run(show_banner=False, transport="http", host="0.0.0.0", port={args.port if args.port else config.mcp_port})''')
-    exec(mcp_script)
-
 def main():
     asyncio.run(main_async())
 
@@ -201,16 +205,18 @@ def display_info(console, info, title=None, border_style=config.color_info_borde
     console.print(info_panel)
     console.print()
 
-def backup_conversation(messages, master_plan, console=None, storage_path=None):
+def backup_conversation(messages, master_plan, console=None, storage_path=None, title=None):
     """Backs up the current conversation to the user's directory."""
     if len(messages) > len(DEFAULT_MESSAGES) and ((not console) or (console and storage_path) or (console and not storage_path and config.backup_required)):
         # determine storage path
         if not storage_path:
             if console:
                 timestamp = getCurrentDateTime()
+                if title:
+                    timestamp += "_"+sanitize_filename(title)[:50].replace(" ", "_")
                 storage_path = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "chats", timestamp)
             else:
-                storage_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+                storage_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), f"temp_{config.backend}")
         # create directory if not exists
         if not os.path.isdir(storage_path):
             Path(storage_path).mkdir(parents=True, exist_ok=True)
@@ -229,8 +235,7 @@ def backup_conversation(messages, master_plan, console=None, storage_path=None):
             console.save_html(html_file, inline_styles=True, theme=MONOKAI)
         # Inform users of the backup location
         if console:
-            info = f"Conversation saved to: {storage_path}\nReport saved to: {html_file}"
-            display_info(console, info)
+            display_info(console, storage_path, title="Backup")
 
 def get_border_style():
     if config.agent_mode:
@@ -349,7 +354,7 @@ async def main_async():
         template_pattern = "|".join(template_list)
         template_pattern = f"""^({template_pattern})"""
 
-        user_request = ""
+        original_request = user_request = ""
         master_plan = ""
         messages = deepcopy(DEFAULT_MESSAGES) # set the tone
 
@@ -362,7 +367,7 @@ async def main_async():
                     TextColumn("[progress.description]{task.description}"),
                     transient=True  # This makes the progress bar disappear after the task is done
                 ) as progress:
-                    task_id = progress.add_task((description if description else "Thinking ...")+" [`Ctrl+C` to cancel]", total=None)
+                    task_id = progress.add_task((description if description else "Thinking ...")+" [Ctrl+C -> Cancel]", total=None)
                     async_task = asyncio.create_task(process())
                     try:
                         while not async_task.done():
@@ -388,9 +393,9 @@ async def main_async():
                 Manages the async task and the progress bar.
                 """
                 if step_number:
-                    print(f"# Starting Step [{step_number}] ... [`Ctrl+C` to cancel]")
+                    print(f"# Starting Step [{step_number}] ... [Ctrl+C -> Cancel]")
                 else:
-                    print(f"# Getting started ... [`Ctrl+C` to cancel]")
+                    print(f"# Getting started ... [Ctrl+C -> Cancel]")
                 # Create the async task but don't await it yet.
                 task = asyncio.create_task(run_tool(tool, tool_instruction))
                 # Await the custom async progress bar that awaits the task.
@@ -400,6 +405,27 @@ async def main_async():
                     task.cancel()
                     await asyncio.sleep(0) # Allow cancellation to propagate
                     raise # Re-raise CancelledError
+            # gnerate title
+            async def generate_title():
+                nonlocal console, original_request
+                if not original_request:
+                    return ""
+                generated_title_output = ""
+                generated_title = ""
+                async def run_prompt_engineering():
+                    nonlocal generated_title_output, generated_title
+                    generated_title_output = agentmake(original_request, system=get_system_generate_title(), **AGENTMAKE_CONFIG)
+                    if generated_title_output:
+                        generated_title = generated_title_output[-1].get("content", "").strip().replace("Title: ", "")
+                try:
+                    await thinking(run_prompt_engineering, "Generating a title ...")
+                    if not generated_title_output:
+                        display_cancel_message(console)
+                except (KeyboardInterrupt, asyncio.CancelledError):
+                    display_cancel_message(console)
+                if generated_title:
+                    set_title(f"BibleMate AI 📝 {generated_title}")
+                return generated_title
 
             if not APP_START and args.exit:
                 break
@@ -418,7 +444,7 @@ async def main_async():
                     # check connection
                     if not config.skip_connection_check:
                         try:
-                            agentmake("Hello!", system=DEFAULT_SYSTEM)
+                            agentmake("Hello!", backend=config.backend, system=DEFAULT_SYSTEM)
                         except Exception as e:
                             print("Connection failed! Please ensure that you have a stable internet connection and that my AI backend and model are properly configured.")
                             print("Viist https://github.com/eliranwong/agentmake#supported-backends for help about the backend configuration.\n")
@@ -466,6 +492,9 @@ async def main_async():
                     user_request = f"//bible/{refs}" if ":" in user_request else f"//chapter/{refs}"
                 else:
                     user_request = f"//search/{user_request}"
+            # direct text response
+            elif user_request.startswith("\\"):
+                user_request = "@get_direct_text_response " + "\n\n" + fix_string(user_request[1:])
             # system commands
             elif user_request.startswith("!"):
                 cmd = user_request[1:].strip()
@@ -671,7 +700,7 @@ async def main_async():
                 else:
                     user_request = f".open {chats_path}"
             elif user_request == ".reload":
-                temp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+                temp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), f"temp_{config.backend}")
                 last_saved_conversation = os.path.join(temp_dir, "conversation.py")
                 if os.path.isfile(last_saved_conversation):
                     user_request = f".import {temp_dir}"
@@ -699,7 +728,10 @@ async def main_async():
                         print("Expected a file or a directory containing `conversation.py` and `master_plan.md`.")
                         os.chdir(cwd)
                         continue
-                    backup_conversation(messages, master_plan, console)
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                     messages = [{"role": i["role"], "content": i["content"]} for i in eval(readTextFile(file_path)) if i.get("role", "") in ("user", "assistant")]
                     if messages:
@@ -723,7 +755,7 @@ async def main_async():
                     if messages:
                         for i in messages:
                             if i.get("role", "") == "user":
-                                display_info(console,Markdown(i['content'].strip()), border_style=get_border_style())
+                                display_info(console, Markdown(i['content'].strip()), border_style=get_border_style())
                             elif i.get("role", "") == "assistant":
                                 console.print(Markdown(i['content'].strip()))
                                 console.print()
@@ -741,7 +773,10 @@ async def main_async():
             # predefined operations with `.` commands
             if user_request in config.action_list:
                 if user_request == ".backup":
-                    backup_conversation(messages, master_plan, console)
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                 elif user_request == ".help":
                     actions = "\n".join([f"- `{k}`: {v}" for k, v in config.action_list.items()])
@@ -1009,10 +1044,15 @@ Press `Ctrl+C` once or twice until the running process is cancelled, while you a
                         write_user_config()
                         display_info(console, f"Default lexicon set to `{config.default_lexicon}`", title="configuration")
                 elif user_request in (".new", ".exit"):
-                    backup_conversation(messages, master_plan, console) # backup
+                    # backup before exit / new
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                 # reset
                 if user_request == ".new":
+                    set_title("BibleMate AI")
                     user_request = ""
                     master_plan = ""
                     messages = deepcopy(DEFAULT_MESSAGES)
@@ -1182,9 +1222,43 @@ Press `Ctrl+C` once or twice until the running process is cancelled, while you a
                         messages = agentmake(messages, system="auto", **AGENTMAKE_CONFIG)
                 messages[-1]["content"] = fix_string(messages[-1]["content"])
 
-            # user specify a single tool
+            # execute a single tool
             if specified_tool and not specified_tool == "@@" and not specified_prompt:
-                display_info(console,Markdown(messages[-1]['content']), border_style=get_border_style())
+                if not specified_tool == "get_direct_text_response":
+                    # refine instruction
+                    refined_instruction_output = []
+                    refined_instruction = ""
+                    async def refine_tool_instruction():
+                        nonlocal refined_instruction_output, refined_instruction, tools, messages, original_request, specified_tool
+                        specified_tool_description = tools.get(specified_tool, "No description available.")
+                        instruction_draft = TOOL_INSTRUCTION_PROMPT + "\n\n# Suggestions\n\n"+messages[-1]['content']+f"\n\n# Tool Description of `{specified_tool}`\n\n"+specified_tool_description+TOOL_INSTRUCTION_SUFFIX
+                        system_tool_instruction = get_system_tool_instruction(specified_tool, specified_tool_description)
+                        if config.light:
+                            this_messages = get_lite_messages(messages, original_request)
+                        else:
+                            this_messages = [{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):]
+                        refined_instruction_output = agentmake(this_messages, system=system_tool_instruction, follow_up_prompt=instruction_draft, **AGENTMAKE_CONFIG)
+                        refined_instruction = refined_instruction_output[-1].get("content", "").strip()
+                    try:
+                        await thinking(refine_tool_instruction, "Refining tool instruction ...")
+                        if not refined_instruction_output:
+                            display_cancel_message(console)
+                            if step == 1:
+                                config.current_prompt = original_request
+                            conversation_broken = True
+                            break
+                        else:
+                            messages[-1]['content'] = refined_instruction
+                    except (KeyboardInterrupt, asyncio.CancelledError):
+                        display_cancel_message(console)
+                        if step == 1:
+                            config.current_prompt = original_request
+                        conversation_broken = True
+                        break
+                    # display refined instruction
+                    display_info(console, Markdown(messages[-1]['content']), title="Refined Instruction", border_style=get_border_style())
+                elif config.prompt_engineering:
+                    display_info(console, Markdown(messages[-1]['content']), title="Refined Instruction", border_style=get_border_style())
                 try:
                     await process_tool(specified_tool, user_request)
                 except (KeyboardInterrupt, asyncio.CancelledError):
@@ -1201,7 +1275,7 @@ Press `Ctrl+C` once or twice until the running process is cancelled, while you a
             # Chat mode
             messages_output = []
             if config.agent_mode is None and not specified_tool == "@@" and not specified_prompt:
-                display_info(console,Markdown(messages[-1]['content']), border_style="none")
+                display_info(console, Markdown(messages[-1]['content']), border_style="none")
                 async def run_chat_mode():
                     nonlocal messages_output, messages, user_request
                     messages_output = agentmake(messages if messages else user_request, system="auto", **AGENTMAKE_CONFIG)
@@ -1387,32 +1461,19 @@ Available tools are: {available_tools}.
                 next_step_output = []
                 next_step = ""
                 async def get_next_step():
-                    nonlocal next_step_output, next_step, next_tool, next_suggestion, tools
+                    nonlocal next_step_output, next_step, next_tool, next_suggestion, tools, messages, original_request
                     if next_tool == "get_direct_text_response":
                         next_step_output = agentmake(next_suggestion, system="biblemate/direct_instruction", **AGENTMAKE_CONFIG)
                         next_step = next_step_output[-1].get("content", "").strip()
                     else:
                         next_tool_description = tools.get(next_tool, "No description available.")
+                        next_suggestion = TOOL_INSTRUCTION_PROMPT + "\n\n# Suggestions\n\n"+next_suggestion+f"\n\n# Tool Description of `{next_tool}`\n\n"+next_tool_description+TOOL_INSTRUCTION_SUFFIX
                         system_tool_instruction = get_system_tool_instruction(next_tool, next_tool_description)
-                        # No context; missing information for the tool from the conversation
-                        #next_step_output = agentmake(next_suggestion, system=system_tool_instruction, **AGENTMAKE_CONFIG)
-                        #next_step = next_step_output[-1].get("content", "").strip()
-                        # Full context, but when a conversation goes long, the agent loses track of the system message.
-                        #next_step = agentmake([{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):], follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
-                        # Minimum context that includes the original request and the latest exchanges, if any
-                        lite_messages = get_lite_messages(messages, original_request)
-                        next_suggestion += f"""
-
-# Remember:
-* Do NOT provide the answer or perform the task. Provide the instruction ONLY, which the AI assistant will follow or answer.
-* You are here to proved the instruction for the current step ONLY.
-* Do not mention the tool name in your instruction.
-* Do not mention further steps or tools to be used after this instruction.
-* Only provide the instruction for the specified tool `{next_tool}`.
-* Pay attention to the information the tool requires and provide the necessary details in your instruction.
-
-You provide the converted instruction directly, without any additional commentary or explanation."""
-                        next_step_output = agentmake(lite_messages, system=system_tool_instruction, follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)
+                        if config.light:
+                            this_messages = get_lite_messages(messages, original_request)
+                        else:
+                            this_messages = [{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):]
+                        next_step_output = agentmake(this_messages, system=system_tool_instruction, follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)
                         next_step = next_step_output[-1].get("content", "").strip()
                 try:
                     await thinking(get_next_step, "Crafting the next instruction ...")
@@ -1511,7 +1572,10 @@ You provide the converted instruction directly, without any additional commentar
             # Backup
             if not conversation_broken:
                 print()
-                backup_conversation(messages, master_plan, console)
+                if config.backup_required:
+                    generated_title = await generate_title()
+                    if generated_title:
+                        backup_conversation(messages, master_plan, console, title=generated_title)
                 config.backup_required = False
     
     # back up configurations
